import commands
import getopt
import osutil
import backup
import gzip_stream
from ReadLineAdapter import FileAdapter
import split_files
import MimeWriter
import sys
import libxml2
import os
import string
import binascii, re, time

from plesk_config import get
from shutil import copyfileobj
from tempfile import mktemp

try: import subprocess
except ImportError: import compat.subprocess as subprocess

def xml_escape(text):
    return text.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")

class ConvertCommand(commands.HelpProvider):
    USAGE_STRING = """convert [<-o|--output> <output-file>] <source-file>"""
    SHORT_HELP_STRING = "Converts old (pre-8.0) dump format to 8.0 dump format"
    HELP_STRING = "Converts old (pre-8.0) dump format to 8.0 dump format"
    def __init__(self, verbosity):
        self.verbosity = verbosity
        self.xsl = get("PRODUCT_ROOT_D") + "/admin/share/backup_convert/backup-convert.xsl"
        self.resFile = None
        os.umask(0077)

    def get_parts(self, file):
        finder = backup.BackupPartsFinder('file', file)
        fh = FileAdapter(split_files.SplitFile(finder))
        if gzip_stream.isGzipped(fh):
            fh = FileAdapter(gzip_stream.GzipReadStream(fh))
        parts = backup.decodeParts(backup.enumerateParts(fh))
        return parts
    
    def parse(self, args):
        if len(args) == 0:
            raise RuntimeError("No backup file specified")
        self.backupFile = args.pop()

        opts, args = getopt.getopt(args, "o:", ["output="])
        for name, value in opts:
            if name in ("-o", "--output"):
                self.resFile = value

    def decode_base64(self, node):
        content = node.getContent()
        if content and content.startswith("==base64"):
            node.setContent(xml_escape(binascii.a2b_base64(content[8:])))

        props = node.get_properties()
        if props:
            for prop in props:
                if not prop.isText():
                    content = prop.getContent()
                    if content and content.startswith("==base64"):
                        prop.setContent(xml_escape(binascii.a2b_base64(content[8:])))

    def encode_base64(self, node):
        content = node.getContent()
        if content:
            node.setContent(binascii.b2a_base64(content).rstrip())
    
    def urldecode(self, data):
        def repl(m): return chr(int(m.group(1), 16))
        return re.sub(r"%([\dA-F]{2})", repl, data.replace("+", " "))

    def act(self):
        if self.resFile:
            outfh = file(self.resFile, "wb")
        else:
            outfh = sys.stdout

        cert_cids = []
        psa_keys = {}

        mime = MimeWriter.MimeWriter(outfh)
        mime.addheader("Mime-Version", "1.0")
        mime.startmultipartbody("related")

        for (partInfo, partContent) in self.get_parts(self.backupFile):
             if partInfo.getheader("Content-Type", "").find("text/xml") >= 0:
                 b = backup.Bundle(fh = partContent, session_dir = None)
                 b.process(osutil.createToFileSaver("/tmp/dump.xml"))
                 b.invalidate()

             if partInfo.getheader("Content-Type", "").find("x-certificate") >= 0:
                 b = backup.Bundle(fh = partContent, session_dir = None)
                 cid = partInfo.getheader("content-id", "")
                 b.process(osutil.createToFileSaver("/tmp/" + cid))
                 b.invalidate()
                 cert_cids.append(cid)

        command = 'xsltproc --output %s %s %s' % ("/tmp/dump-converted.xml", self.xsl, "/tmp/dump.xml")
        os.system(command)

        convertDoc = libxml2.parseFile("/tmp/dump-converted.xml")
        root = convertDoc.children
        for child in root:
            if child.name == "certificate" and child.hasProp("cid"):
                cid = child.prop("cid")
                if cid in cert_cids:
                    fh = file("/tmp/" + cid, "rb")
                    child.newChild(None, "certificate-data", self.urldecode(fh.read()))
                    fh.close()

                child.unsetProp("cid")

                if child.hasProp("cid-ca"):
                   cid = child.prop("cid-ca")
                   if cid in cert_cids:
                       fh = file("/tmp/" + cid, "rb")  
                       child.newChild(None, "ca-certificate", self.urldecode(fh.read()))
                       fh.close()

                   child.unsetProp("cid-ca")

                if child.hasProp("cid-private"):
                   cid = child.prop("cid-private")
                   if cid in cert_cids:
                       fh = file("/tmp/" + cid, "rb")  
                       child.newChild(None, "private-key", self.urldecode(fh.read()))
                       fh.close()

                   child.unsetProp("cid-private")

            elif child.name == "key":
                cid = "plsk00000000000" + str(len(psa_keys)) + ".key"
                child.newProp("cid", cid)
                psa_keys[cid] = child.getContent()
                child.setContent(None)

            elif child.name == "limit" and child.prop("name") == "expiration" and child.getContent() != "-1":
                child.setContent(time.strftime("%Y-%m-%d", time.localtime(int(child.getContent()))))

            if not child.isText():
                self.decode_base64(child)

                if (child.name == "domain" or child.name == "subdomain") and child.prop("name").find("xn--") >= 0:
                    import encodings.idna

                    parts = child.prop("name").split(".")
                    name = ""

                    for p in parts:
                        if p[0:4] == "xn--":
                            p = encodings.idna.ToUnicode(p)
                        name = name + p + "."

                    child.setProp("name", name[:-1].encode("UTF-8"))

                elif child.name == "autoresponder" and child.hasProp("subject"):
                    self.encode_base64(child.hasProp("subject"))

                elif child.name == "text" and child.get_parent() and child.get_parent().name == "autoresponder":
                    self.encode_base64(child)

        convertDoc.saveFile("/tmp/dump-converted.xml") 

        for (part_info, part_content) in self.get_parts(self.backupFile):
            type = part_info.getheader("Content-Encoding","")
            contentId = part_info.getheader("content-id", "")

            if contentId == "":
                contentId = "dump.xml"

            if contentId in cert_cids:
                continue

            part = mime.nextpart()
            part.addheader("Content-Disposition", "inline; filename=" + contentId)
            part.addheader("Content-Id", contentId)
            part.startbody("application/octet-stream")

            if type == "gzip":
                part_content = FileAdapter(gzip_stream.GzipReadStream(part_content))
             
            hdr = part_info.getheader("Content-Type", "")

            if hdr.find("text/xml") >= 0:
                convertXml = file("/tmp/dump-converted.xml", "rb")
                outfh.write(convertXml.read())
                convertXml.close()
                osutil.unlink_nothrow("/tmp/dump-converted.xml")
                osutil.unlink_nothrow("/tmp/dump.xml")

            elif hdr.find("x-war-webapp") >= 0:
                # make tar
                m = re.search('name="(.+?)"', hdr)
                if m: name = m.group(1)
                else: name = 'webapp.war'

                osutil.saveToFile("/tmp/" + name, part_content)
                tar = os.popen(get("TAR_BIN") + " -f - -c -C /tmp " + name, "r")
                copyfileobj(tar, outfh)
                tar.close()
                osutil.unlink_nothrow("/tmp/" + name)

            else:
                copyfileobj(part_content, outfh)

        for (cid, content) in psa_keys.items():
            part = mime.nextpart()
            part.addheader("Content-Disposition: inline; filename=", cid)
            part.addheader("Content-Id", cid)
            part.startbody("application/octet-stream")
            outfh.write(content)

        mime.lastpart()
        outfh.close()

        for cid in cert_cids:
            osutil.unlink_nothrow("/tmp/" + cid)

commands.register("convert", ConvertCommand, 2)
