import signal
import logging
from xml.dom import minidom
from xml.parsers.expat import ExpatError

import osutil
import subproc

import pmm_config
from pmmcli_exceptions import PMMUtilityException
from pmm_task import DictionaryTask, InvalidTaskOperation, StoppedTaskStatusCreator, BaseDictionaryTaskHandler, getPMMTaskManager
from pmm_api_xml_protocols import TaskStatus, Response, Data, TaskId, Stopped, Working, Starting


_logger = logging.getLogger("pmmcli.migration_handler")


class MigrationTaskHandler(BaseDictionaryTaskHandler):
    def __init__(self):
        BaseDictionaryTaskHandler.__init__(self,'Migration')

    def stop_task(self, task):
        if not isinstance(task, MigrationTask):
            raise InvalidTaskOperation, 'MigrationTaskHandler does not bind with MigrationTask instance (%s)' % task.__class__.__name__
        if not task.get_stopped():
            try:
                if task.is_active():
                    osutil.kill(task.get_os_pid(), signal.SIGTERM)
                    _logger.warning("migration task '%s' is stopped" % task.get_task_id())
            except OSError, e:
                _logger.warning("stop_task '%s' failed: %s" % (task.get_task_id(), str(e)))
            migration_session_id = task.get('session_id')
            errcode = MigrationHandler.migrationStopTask(migration_session_id)[1]
            if errcode != 0:
                _logger.warning("stop_task '%s' failed: Errcode is %s" % (task.get_task_id(), errcode))

    def remove_task(self, task):
        if not isinstance(task, MigrationTask):
            raise InvalidTaskOperation, 'MigrationTaskHandler does not bind with MigrationTask instance (%s)' % task.__class__.__name__
        
        migration_session_id = task.get('session_id')
        
        errcode = MigrationHandler.migrationRemoveSessionData(migration_session_id)[1]
        if errcode != 0:
            _logger.warning("remove_task '%s' failed: Errcode is %s" % (task.get_task_id(), errcode))
        
        self.__removeTaskChildren(task)

    def __removeTaskChildren(self, task):
        _logger.debug("Remove children tasks for task %s" % task.get_task_id())
        
        task_manager = getPMMTaskManager()
        
        task_id = task.get("restore_task_id")
        if task_id is not None:
            _logger.debug("Remove child task '%s'" % str(task_id))
            task_manager.removeTask(task_id)
        else:
            _logger.warning("No child tasks for task '%s'" % str(task.get_task_id()))

    def get_status(self, task):
        if not isinstance(task, MigrationTask):
            raise InvalidTaskOperation, 'MigrationTaskHandler does not bind with MigrationTask instance (%s)' % task.__class__.__name__
        if task.get_stopped():
            return StoppedTaskStatusCreator(task).response()
        else:
            return MigrationTaskStatusCreator(task).response()

    def get_log(self, task):
        if not isinstance(task, MigrationTask):
            raise InvalidTaskOperation, 'MigrationTaskHandler does not bind with MigrationTask instance (%s)' % task.__class__.__name__
        return MigrationTaskLogCreator(task).response()


class MigrationTask(DictionaryTask):
    def __init__(self, cmd, pid, dictionary = {}, task_handler = MigrationTaskHandler()):
        DictionaryTask.__init__(self, cmd, pid, dictionary, task_handler)

    def required_keys():
        return ["session_id",
                "additional_info",
                "restore_task_id"]

    required_keys = staticmethod(required_keys)


class MigrationTaskStatusCreator:
    def __init__(self, task):
        if not isinstance(task, MigrationTask):
            raise InvalidTaskOperation, 'MigrationTaskStatusCreator does not bind with MigrationTask instance (%s)' % task.__class__.__name__
        self.__task = task

    def response(self):
        if not MigrationHandler.installed():
            return TaskStatus( stopped = Stopped() )
        
        migration_session_id = self.__task.get('session_id')
        try:
            data, errcode, message = MigrationHandler.migrationGetTaskStatus(migration_session_id)
            if errcode == 0:
                task_status = data.get_task_status()
                if task_status is not None:
                    # Should remove task_id attribute from task_status.
                    # Child backup task of migration task does not have task_id.
                    backup_status = task_status.get_mixed().get_backup()
                    backup_status.set_task_id( None )
                    # Failed restore task of migration task does not have task_id also.
                    restore_status = task_status.get_mixed().get_restore()
                    if restore_status is not None:
                        if restore_status.get_task_id() == '':
                            restore_status.set_task_id( None )
                    
                    return task_status
            _logger.warning("could not get task status on task '%s'" % self.__task.get_task_id())
        except PMMUtilityException, e:
            _logger.warning("error when try to get task status on task '%s': %s" % (self.__task.get_task_id(), str(e)))
        return TaskStatus (working = Working( starting = Starting()))


class MigrationTaskLogCreator:
    def __init__(self, task):
        if not isinstance(task, MigrationTask):
            raise InvalidTaskOperation, 'MigrationTaskLogCreator does not bind with MigrationTask instance (%s)' % task.__class__.__name__
        self.__task = task

    def response(self):
        if not MigrationHandler.installed():
            return "Migration handler isn't installed"
        
        migration_session_id = self.__task.get('session_id')
        try:
            data, errcode, message = MigrationHandler.migrationGetTaskLog(migration_session_id)
            if errcode == 0:
                task_log = data.get_task_log()
                if task_log is not None:
                    return task_log
            _logger.warning("could not get task log on task '%s'" % self.__task.get_task_id())
        except PMMUtilityException, e:
            _logger.warning("error when try to get task log on task '%s': %s" % (self.__task.get_task_id(), str(e)))
        return 'Failed to get task log'


# class MigrationHandler
# interface class for migration handler calls
# see migration_handler.py command-line interface
class MigrationHandler:
    def installed():
        return pmm_config.migration_handler_installed()

    installed = staticmethod(installed)

    def migrationGetScoutInfo(migration_task_description):
        cmd = subproc.CmdLine(pmm_config.migration_handler(), stdin = migration_task_description)
        for migration_handler_arg in pmm_config.migration_handler_args():
            cmd.arg(migration_handler_arg)
        
        cmd.arg('--migration-get-scout-info')
        
        try:
            proc = cmd.spawn()
        except subproc.NonzeroExitException, x:
            raise PMMUtilityException('migration_handler', x)
        
        response = Response.factory()
        try:
            response.build(minidom.parseString(proc.stdout.encode('utf-8')).childNodes[0])
        except ExpatError, x:
            errcode = 1000
            message =  u"pmm utility 'migration_handler' error: Xml reading error: " + unicode(x)
            return None, errcode, message
        
        response_data_object = response.get_data()
        
        if response.get_errcode() != '0':
            errcode_str, message = MigrationHandler.prepareMigrationhandlerResult(response)
            try:
                errcode = int(errcode_str)
            except (TypeError, ValueError), x:
                errcode = 1000
                message = "pmm utility 'migration_handler' error: Xml reading error: Wrong format for errcode_str: " + str(x)
            return response_data_object, errcode, message 
        
        if response_data_object is None:
            errcode = 1000
            message =  u"pmm utility 'migration_handler' error: Xml reading error: " + unicode(x)
            return None, errcode, message
        
        response_scout_result_object = response_data_object.get_scout_result()
        if response_scout_result_object is None:
            errcode = 1000
            message =  u"pmm utility 'migration_handler' error: Xml reading error: " + unicode(x)
            return None, errcode, message
        
        data = Data.factory( scout_result = response_scout_result_object )
        
        return data, 0, ''

    migrationGetScoutInfo = staticmethod(migrationGetScoutInfo)

    def migrationGetObjectsList(migration_session_id, agent_to_use):
        return MigrationHandler.callMigrationHandlerWithResponse('--migration-get-objects-list', session_id = migration_session_id, input_str = agent_to_use)

    migrationGetObjectsList = staticmethod(migrationGetObjectsList)

    def migrationSetIPMapping(migration_session_id, ip_mapping):
        return MigrationHandler.callMigrationHandlerDatalessly('--migration-set-ip-mapping', session_id = migration_session_id, input_str = ip_mapping)

    migrationSetIPMapping = staticmethod(migrationSetIPMapping)

    def migrationGetIPMapping(migration_session_id):
        return MigrationHandler.callMigrationHandlerWithResponse('--migration-get-ip-mapping', session_id = migration_session_id )

    migrationGetIPMapping = staticmethod(migrationGetIPMapping)

    def migrationGetIPForMapping(migration_session_id):
        return MigrationHandler.callMigrationHandlerWithResponse('--migration-get-ip-for-mapping', session_id = migration_session_id )

    migrationGetIPForMapping = staticmethod(migrationGetIPForMapping)

    def migrationSetDBMapping(migration_session_id, db_mapping):
        return MigrationHandler.callMigrationHandlerDatalessly('--migration-set-db-mapping', session_id = migration_session_id, input_str = db_mapping)

    migrationSetDBMapping = staticmethod(migrationSetDBMapping)

    def migrationGetDBMapping(migration_session_id):
        return MigrationHandler.callMigrationHandlerWithResponse('--migration-get-db-mapping', session_id = migration_session_id)

    migrationGetDBMapping = staticmethod(migrationGetDBMapping)

    def migrationSetOwnerMapping(migration_session_id, owner_mapping):
        return MigrationHandler.callMigrationHandlerDatalessly('--migration-set-owner-mapping', session_id = migration_session_id, input_str = owner_mapping)

    migrationSetOwnerMapping = staticmethod(migrationSetOwnerMapping)

    def migrationGetOwnerMapping(migration_session_id):
        return MigrationHandler.callMigrationHandlerWithResponse('--migration-get-owner-mapping', session_id = migration_session_id)

    migrationGetOwnerMapping = staticmethod(migrationGetOwnerMapping)

    def migrationGetDstTemporaryDirectory(migration_session_id):
        return MigrationHandler.callMigrationHandlerWithResponse('--migration-get-dst-temporary-directory', session_id = migration_session_id)

    migrationGetDstTemporaryDirectory = staticmethod(migrationGetDstTemporaryDirectory)

    def migrationSetPolicy(migration_session_id, policy):
        return MigrationHandler.callMigrationHandlerDatalessly('--migration-set-policy', session_id = migration_session_id, input_str = policy)

    migrationSetPolicy = staticmethod(migrationSetPolicy)

    def migrationGetPolicy(migration_session_id):
        return MigrationHandler.callMigrationHandlerWithResponse('--migration-get-policy', session_id = migration_session_id)

    migrationGetPolicy = staticmethod(migrationGetPolicy)

    def migrationGetSelectedObjects(migration_session_id):
        return MigrationHandler.callMigrationHandlerWithResponse('--migration-get-selected-objects', session_id = migration_session_id)

    migrationGetSelectedObjects = staticmethod(migrationGetSelectedObjects)

    def migrationGetSessionObjectsList(migration_session_id):
        return MigrationHandler.callMigrationHandlerWithResponse('--migration-get-session-objects-list', session_id = migration_session_id)

    migrationGetSessionObjectsList = staticmethod(migrationGetSessionObjectsList)

    def migrationGetAgentCredentials(migration_session_id):
        return MigrationHandler.callMigrationHandlerWithResponse('--migration-get-agent-credentials', session_id = migration_session_id)

    migrationGetAgentCredentials = staticmethod(migrationGetAgentCredentials)

    def migrationGetIPForMappingObjects(migration_session_id, selected_objects ):
        cmd = subproc.CmdLine(pmm_config.migration_handler(), stdin = selected_objects)
        for migration_handler_arg in pmm_config.migration_handler_args():
            cmd.arg(migration_handler_arg)
        
        cmd.arg('--migration-get-ip-for-mapping-objects')
        cmd.arg(migration_session_id)
        
        errcode = 0
        message = None
        data = None
        try:
            proc = cmd.spawn()
            responce = Response.factory()
            responce.build(minidom.parseString(proc.stdout.encode('utf-8')).childNodes[0])
            data = Data.factory()
            if responce.data is not None:
                data.migration_ip_mapping = responce.data.migration_ip_mapping
            if responce.errcode != '0':
                errcode_str, message = MigrationHandler.prepareMigrationhandlerResult(responce)
                try:
                    errcode = int(errcode_str)
                except (TypeError, ValueError), x:
                    errcode = 1000
                    message = "pmm utility 'migration_handler' error: invalid errcode format (" + unicode(errcode) + ")"
        
        except subproc.NonzeroExitException, x:
            raise PMMUtilityException('migration_handler', x)
        except ExpatError, x:
            errcode = 1000
            message =  u"pmm utility 'migration_handler' error: Xml reading error: " + unicode(x)
        
        return data, errcode, message

    migrationGetIPForMappingObjects = staticmethod(migrationGetIPForMappingObjects)

    def migrationStart(migration_session_id):
        cmd = subproc.CmdLine(pmm_config.migration_handler())
        for migration_handler_arg in pmm_config.migration_handler_args():
            cmd.arg(migration_handler_arg)
        
        migration_task_start_arg = '--migration-task-start'
        cmd.arg(migration_task_start_arg)
        
        migration_session_id_arg = migration_session_id
        cmd.arg(migration_session_id_arg)
        
        try:
            proc = cmd.spawn()
            responce = Response.factory()
            responce.build(minidom.parseString(proc.stdout.encode('utf-8')).childNodes[0])
        except subproc.NonzeroExitException, x:
            raise PMMUtilityException('migration_handler', x)
        except ExpatError, x:
            message =  u"pmm utility 'migration_handler' error: Xml reading error: " + unicode(x)
            return None, 1000, message
        
        errcode_str = responce.get_errcode()
        errmsg = responce.get_errmsg()
        data = responce.get_data()
        
        if errcode_str != '0':
            try:
                errcode = int(errcode_str)
            except (TypeError, ValueError), x:
                errcode = 1000
                message = "pmm utility 'migration_handler' error: invalid errcode format (" + unicode(errcode_str) + ")"
            return data, errcode, errmsg
        # errcode 0 specifies operation is successfully ended, i.e. migration task is started
        if errmsg is None or errmsg == '':
            message = u"pmm utility 'migration_handler' error: invalid migration task cmd format"
            return None, 1000, message
        if data is None or data.get_task_list() is None or data.get_task_list().get_task() is None:
            message = u"pmm utility 'migration_handler' error: invalid responce format (no task list)"
            return None, 1000, message
        if len(data.get_task_list().get_task()) < 1:
            message = u"pmm utility 'migration_handler' error: invalid responce format (no tasks in list)"
            return None, 1000, message
        
        task = data.get_task_list().get_task()[0]
        if task is None:
            message = u"pmm utility 'migration_handler' error: invalid responce format (task is None)"
            return None, 1000, message
        
        task_pid = task.get_task_id()
        task_pid_value = 0
        try:
            task_pid_value = int(task_pid)
        except (ValueError, TypeError), e:
            message = u"pmm utility 'migration_handler' error: invalid task_pid format (" + unicode(task_pid) + "): " + str(e)
            return None, 1000, message
        
        cmdline = errmsg
        additional_info = task.get_additional_info()
        if additional_info is None:
            message =  u"pmm utility 'migration_handler' error: invalid responce format (empty additional info)"
            return None, 1000, message
        
        task = MigrationTask( cmdline, task_pid_value, 
                              {'session_id':migration_session_id, 
                               'additional_info':additional_info})
        task_id_object = TaskId()
        task_id_object.setValueOf_(task.get_task_id())
        data_object = Data.factory( task_id = task_id_object )
        return data_object, 0, None

    migrationStart = staticmethod(migrationStart)

    def migrationGetTaskStatus(migration_session_id):
        cmd = subproc.CmdLine(pmm_config.migration_handler())
        for migration_handler_arg in pmm_config.migration_handler_args():
            cmd.arg(migration_handler_arg)
        
        cmd.arg('--migration-get-task-status')
        cmd.arg(migration_session_id)
        
        errcode = 0
        message = None
        data = None
        try:
            proc = cmd.spawn()
            responce = Response.factory()
            responce.build(minidom.parseString(proc.stdout.encode('utf-8')).childNodes[0])
            data = Data.factory()
            if responce.data is not None:
                data.task_status = responce.data.task_status
            if responce.errcode != '0':
                errcode_str, message = MigrationHandler.prepareMigrationhandlerResult(responce)
                try:
                    errcode = int(errcode_str)
                except (TypeError, ValueError), x:
                    errcode = 1000
                    message = "pmm utility 'migration_handler' error: invalid errcode format (" + unicode(errcode_str) + ")"
        except subproc.NonzeroExitException, x:
            raise PMMUtilityException('migration_handler', x)
        except ExpatError, x:
            errcode = 1000
            message =  u"pmm utility 'migration_handler' error: Xml reading error: " + unicode(x)
        return data, errcode, message

    migrationGetTaskStatus = staticmethod(migrationGetTaskStatus)

    def migrationGetTaskLog(migration_session_id):
        cmd = subproc.CmdLine(pmm_config.migration_handler())
        for migration_handler_arg in pmm_config.migration_handler_args():
            cmd.arg(migration_handler_arg)
        
        cmd.arg('--migration-get-task-log')
        cmd.arg(migration_session_id)
        
        errcode = 0
        message = None
        data = None
        try:
            proc = cmd.spawn()
            responce = Response.factory()
            responce.build(minidom.parseString(proc.stdout.encode('utf-8')).childNodes[0])
            data = Data.factory()
            if responce.data is not None:
                data.task_log = responce.data.task_log
            if responce.errcode != '0':
                errcode_str, message = MigrationHandler.prepareMigrationhandlerResult(responce)
                try:
                    errcode = int(errcode_str)
                except (TypeError, ValueError), x:
                    errcode = 1000
                    message = "pmm utility 'migration_handler' error: invalid errcode format (" + unicode(errcode_str) + ")"
        except subproc.NonzeroExitException, x:
            raise PMMUtilityException('migration_handler', x)
        except ExpatError, x:
            errcode = 1000
            message =  u"pmm utility 'migration_handler' error: Xml reading error: " + unicode(x)
        return data, errcode, message

    migrationGetTaskLog = staticmethod(migrationGetTaskLog)

    def migrationRemoveSessionData(migration_session_id):
        return MigrationHandler.callMigrationHandlerDatalessly('--migration-remove-session-data', migration_session_id)

    migrationRemoveSessionData = staticmethod(migrationRemoveSessionData)

    def migrationStopTask(migration_session_id):
        return MigrationHandler.callMigrationHandlerDatalessly('--migration-stop-task', migration_session_id)

    migrationStopTask = staticmethod(migrationStopTask)

    def migrationGetSessionList():
        return MigrationHandler.callMigrationHandlerWithResponse('--migration-get-sessions-list', None)

    migrationGetSessionList = staticmethod(migrationGetSessionList)

    def migrationGetSession(migration_session_id):
        data, errcode, message = MigrationHandler.callMigrationHandlerWithResponse('--migration-get-session', migration_session_id)
        if errcode != 0:
            return data, errcode, message
        if data is not None:
            session_list_object = data.get_session_list()
            if session_list_object is not None:
                if len(session_list_object.get_session()) == 1:
                    return data, errcode, message
        errcode = 1000
        message =  u"pmm utility 'migration_handler' error: wrong format --migration-get-session output"
        return None, errcode, message

    migrationGetSession = staticmethod(migrationGetSession)

    def callMigrationHandlerWithResponse(command, session_id = None, input_str = ''):
        cmd = subproc.CmdLine(pmm_config.migration_handler(), stdin = input_str)
        for migration_handler_arg in pmm_config.migration_handler_args():
            cmd.arg(migration_handler_arg)
        
        cmd.arg(command)
        if session_id is not None:
            cmd.arg(session_id)
        
        errcode = 0
        message = None
        data = None
        try:
            proc = cmd.spawn()
            responce = Response.factory()
            responce.build(minidom.parseString(proc.stdout.encode('utf-8')).childNodes[0])
            data = responce.data
            if responce.errcode != '0':
                errcode_str, message = MigrationHandler.prepareMigrationhandlerResult(responce)
                try:
                    errcode = int(errcode_str)
                except (TypeError, ValueError), x:
                    errcode = 1000
                    message = "pmm utility 'migration_handler' error: invalid errcode format (" + unicode(errcode_str) + ")"
        except subproc.NonzeroExitException, x:
            raise PMMUtilityException('migration_handler', x)
        except ExpatError, x:
            errcode = 1000
            message =  u"pmm utility 'migration_handler' error: Xml reading error: " + unicode(x)
        return data, errcode, message

    callMigrationHandlerWithResponse = staticmethod(callMigrationHandlerWithResponse)

    def callMigrationHandlerDatalessly(command, session_id = None, input_str = ''):
        cmd = subproc.CmdLine(pmm_config.migration_handler(), stdin = input_str)
        for migration_handler_arg in pmm_config.migration_handler_args():
            cmd.arg(migration_handler_arg)
        
        cmd.arg(command)
        if session_id is not None:
            cmd.arg(session_id)
        
        errcode = 0
        message = None
        try:
            proc = cmd.spawn()
            responce = Response.factory()
            responce.build(minidom.parseString(proc.stdout.encode('utf-8')).childNodes[0])
            if responce.errcode != '0':
                errcode_str, message = MigrationHandler.prepareMigrationhandlerResult(responce)
                try:
                    errcode = int(errcode_str)
                except (TypeError, ValueError), x:
                    errcode = 1000
                    message = "pmm utility 'migration_handler' error: invalid errcode format (" + unicode(errcode_str) + ")"
        
        except subproc.NonzeroExitException, x:
            raise PMMUtilityException('migration_handler', x)
        except ExpatError, x:
            errcode = 1000
            message =  u"pmm utility 'migration_handler' error: Xml reading error: " + unicode(x)
        
        return None, errcode, message

    callMigrationHandlerDatalessly = staticmethod(callMigrationHandlerDatalessly)

    def prepareMigrationhandlerResult(responce):
        return responce.errcode, responce.errmsg

    prepareMigrationhandlerResult = staticmethod(prepareMigrationhandlerResult)

