import os
import time
import shutil
import osutil
import signal
import logging
from xml.dom import minidom
from threading import RLock, currentThread
from cPickle import Pickler, Unpickler, UnpicklingError, HIGHEST_PROTOCOL

import pmm_config
import pmm_dump_formatter
import pmmcli_daemon_service
import pmm_dump_access_service
from pmm_api_xml_protocols import TaskStatus, Stopped, Finished, TaskLog, WorkingProgress, Working, Starting, TaskList
from pmm_api_xml_protocols import Task as TaskElement

import validator
from execution_result import ExecutionResult


_logger = logging.getLogger("pmmcli.task")


class FileDumpMarshaller:
    def __init__(self, filename):
        self.__filename = filename
        self.__storage = open(self.__filename,'wb')
        self.__pickler = Pickler(self.__storage, HIGHEST_PROTOCOL)

    def dump(self, data):
        self.__pickler.dump(data)

    def close(self):
        self.__storage.close()

    def filename(self):
        return self.__filename

    def storage(self):
        return self.__storage


class FileLoadMarshaller:
    def __init__(self,filename):
        self.__filename = filename
        self.__storage = open(self.__filename,'rb')
        self.__unpickler = Unpickler(self.__storage)

    def load(self):
        loadeddata = self.__unpickler.load()
        return loadeddata

    def close(self):
        self.__storage.close()

    def filename(self):
        return self.__filename

    def storage(self):
        return self.__storage


class InvalidTaskOperation(Exception):
    "Exception to indicate an invalid operation on task"


class TaskHandler:
    def __init__(self, task_type):
        self.__task_type = task_type

    def get_task_type(self):
        return self.__task_type

    def stop_task(self, task):
        if not isinstance(task, Task):
            raise InvalidTaskOperation, 'TaskHandler does not bind with Task instance (%s)' % task.__class__.__name__
        if not task.get_stopped():
            try:
                if task.is_active():
                    osutil.kill(task.get_os_pid(), signal.SIGTERM)
            except OSError, e:
                _logger.warning("stop_task on task %s failed: %s" % (task.get_task_id(), str(e)))

    def remove_task(self, task):
        if not isinstance(task, Task):
            raise InvalidTaskOperation, 'TaskHandler does not bind with Task instance (%s)' % task.__class__.__name__
        # remove task-specific additional data here
        # this method is invoked from the task manager, that implements add/remove task operations
        # There is no additional data for tasks types, being processed with TaskHandler class

    def get_status(self, task):
        if not isinstance(task, Task):
            raise InvalidTaskOperation, 'TaskHandler does not bind with Task instance (%s)' % task.__class__.__name__
        if task.get_stopped():
            return StoppedTaskStatusCreator(task).response()
        else:
            active = task.is_active()
            if active:
                return RunningTaskStatusCreator(task).response()
            else:
                return FinishedTaskStatusCreator(task).response()

    def get_log(self, task):
        if not isinstance(task, Task):
            raise InvalidTaskOperation, 'TaskHandler does not bind with Task instance (%s)' % task.__class__.__name__
        # Have to be overwritten in inherited classes
        return ''


class BaseDictionaryTaskHandler(TaskHandler):
    def __init__(self, name = 'None'):
        TaskHandler.__init__(self,name)

    def get_log(self, task):
        if not isinstance(task, BaseDictionaryTask):
            raise InvalidTaskOperation, 'BaseDictionaryTaskHandler does not bind with BaseDictionaryTask instance (%s)' % task.__class__.__name__
        try:
            return BaseDictionaryTaskLogCreator(task).response()
        except InvalidTaskOperation, x:
            _logger.warning("Exception raised in BaseDictionaryTaskHandler: %s" % str(x))
            return TaskHandler.get_log(self,task)


class BackupTaskHandler(BaseDictionaryTaskHandler):
    def __init__(self):
        BaseDictionaryTaskHandler.__init__(self,'Backup')

    def remove_task(self, task):
        if not isinstance(task, BackupTask):
            raise InvalidTaskOperation, 'BackupTaskHandler does not bind with BackupTask instance (%s)' % task.__class__.__name__
        task_dir = task.get('session_path')
        if os.path.isdir(task_dir):
            osutil.unlink_recursively(task_dir)

    def get_status(self, task):
        if not isinstance(task, BackupTask):
            raise InvalidTaskOperation, 'BackupTaskHandler does not bind with BackupTask instance (%s)' % task.__class__.__name__
        if task.get_stopped():
            return StoppedTaskStatusCreator(task).response()
        else:
            active = task.is_active()
            if active:
                return RunningBackupTaskStatusCreator(task).response()
            else:
                return FinishedBackupTaskStatusCreator(task).response()


class RestoreTaskHandler(BaseDictionaryTaskHandler):
    def __init__(self):
        BaseDictionaryTaskHandler.__init__(self,'Restore')

    def remove_task(self, task):
        if not isinstance(task, RestoreTask):
            raise InvalidTaskOperation, 'RestoreTaskHandler does not bind with RestoreTask instance (%s)' % task.__class__.__name__
        task_dir = task.get('session_path')
        if os.path.isdir(task_dir):
            osutil.unlink_recursively(task_dir)

    def get_status(self, task):
        if not isinstance(task, RestoreTask):
            raise InvalidTaskOperation, 'RestoreTaskHandler does not bind with RestoreTask instance (%s)' % task.__class__.__name__
        if task.get_stopped():
            return StoppedTaskStatusCreator(task).response()
        else:
            active = task.is_active()
            if active:
                return RunningRestoreTaskStatusCreator(task).response()
            else:
                return FinishedRestoreTaskStatusCreator(task).response()


class Task:
    def __init__(self, cmd, pid, task_handler = TaskHandler("None")):
        self.__task_handler = task_handler
        self.__tasktype = str(task_handler.get_task_type())
        self.__cmd = cmd
        self.__os_pid = int(pid)
        self.__task_id = None
        self.__creation_date = time.time()
        self.__stopped = False
        self.__removed = False
        getPMMTaskManager().addTask(self)

    def get_task_handler(self):
        return self.__task_handler

    def dump(self, dump_marshaller):
        if self.__removed:
            raise InvalidTaskOperation, 'Could not dump removed task'
        pdump = dump_marshaller.dump
        pdump(self)

    def load(load_marshaller):
        pload = load_marshaller.load
        return pload()

    load = staticmethod(load)

    def is_active(self):
        active = osutil.is_active(self.__os_pid)
        same_cmd = False
        if active:
            process_cmd = osutil.get_cmd(self.__os_pid)
            # This deal with 'get_cmd' implementation differences on Windows and Linux platforms
            same_cmd = self.__cmd.startswith(process_cmd)
            if not same_cmd:
                _logger.debug("process (%s) found with the same pid as %s task pid (%d)" % (process_cmd, self.__tasktype, self.__os_pid))
        return same_cmd

    def stop(self):
        self.__task_handler.stop_task(self)
        self.__stopped = True

    def remove(self):
        _logger.debug("Remove task %s" % self.__task_id)
        self.__task_handler.remove_task(self)
        self.__removed = True

    def get_status(self):
        status = self.__task_handler.get_status(self)
        if not isinstance(status, TaskStatus):
            raise InvalidTaskOperation, "TaskStatus returned (%s) is not TaskStatus instance" % status.__class__.__name__
        status.set_task_id(self.__task_id)
        return status

    def get_log(self):
        return self.__task_handler.get_log(self)

    def get_os_pid(self):
        return self.__os_pid

    def get_task_id(self):
        return self.__task_id

    def get_tasktype(self):
        return self.__tasktype

    def get_creation_date_formatted(self):
        return time.strftime('%Y-%b-%d %H:%M:%S',time.localtime(self.__creation_date))

    def get_creation_date(self):
        return self.__creation_date

    def get_stopped(self):
        return self.__stopped

    def set_task_id(self, task_id):
        self.__task_id = task_id


class BaseDictionaryTask(Task):
    def __init__(self, cmd, pid, dictionary = {}, task_handler = BaseDictionaryTaskHandler("None")):
        self.__dict = dictionary
        Task.__init__(self, cmd, pid, task_handler)

    def get(self, key_name):
        return self.__dict[key_name]

    def set(self, key_name, key_value):
        self.__dict[key_name] = key_value

    def keys(self):
        return self.__dict.keys()

    def has_key(self, key_name):
        return self.__dict.has_key(key_name)


class DictionaryTask(BaseDictionaryTask):
    def __init__(self, cmd, pid, dictionary = {}, task_handler = TaskHandler("None")):
        for key in self.required_keys():
            if not dictionary.has_key(key):
                dictionary[key] = None
        BaseDictionaryTask.__init__(self, cmd, pid, dictionary, task_handler)

    def required_keys():
        return []

    required_keys = staticmethod(required_keys)


class BackupTask(DictionaryTask):
    def __init__(self, cmd, pid, dictionary = {}, task_handler = BackupTaskHandler()):
        DictionaryTask.__init__(self, cmd, pid, dictionary, task_handler)
        self.set('log_filename', os.path.join(self.get('session_path'),'psadump.log'))
        self.set('migration_result_filename', os.path.join(self.get('session_path'),'migration.result'))
        getPMMTaskManager().updateTask(self)

    def required_keys():
        return ["owner_guid",
                "owner_type",
                "dumps_storage_credentials",
                "fullname",
                "backup_profile_name",
                "mailto",
                "topobject_id",
                "topobject_type",
                "mailsent",
                "host_credentials",
                "log_filename",
                "migration_result_filename",
                "session_path"]

    required_keys = staticmethod(required_keys)


class RestoreTask(DictionaryTask):
    def __init__(self, cmd, pid, dictionary = {}, task_handler = RestoreTaskHandler()):
        DictionaryTask.__init__(self, cmd, pid, dictionary, task_handler)
        self.set('log_filename', os.path.join(self.get('session_path'),'migration.log'))
        self.set('migration_result_filename', os.path.join(self.get('session_path'),'migration.result'))
        getPMMTaskManager().updateTask(self)

    def required_keys():
        return ["owner_guid",
                "owner_type",
                "dumps_storage_credentials",
                "fullname",
                "backup_profile_name",
                "mailto",
                "mailsent",
                "host_credentials",
                "log_filename",
                "migration_result_filename",
                "session_path"]

    required_keys = staticmethod(required_keys)


execution_result_validator = None


def getExecutionResultValidator():
    global execution_result_validator
    if not execution_result_validator:
        execution_result_validator = XsdValidator(pmm_config.execution_result_schema())
    return execution_result_validator


class XsdValidator:
    def __init__(self, schema):
        self.__validator = validator.Validator(schema)

    def do(self, xml):
        return self.__validator.do(xml)


class PMMTaskPersistentStorage(object):
    def __init__(self):
        self.accessMutex = RLock()
        self.persistent_storage_dir = pmm_config.tasks_dir()

    def __make_persistent_storage(self):
        tasksDirectory = self.persistent_storage_dir
        if not os.path.isdir(tasksDirectory):
            os.mkdir(tasksDirectory)

    def __loadTask(self, fullname):
        loader = FileLoadMarshaller(fullname)
        task_item = None
        try:
            task_item = Task.load(loader)
            loader.close()
        except (AttributeError, ValueError, EOFError, ImportError, IndexError, UnpicklingError), e:
            _logger.error("Could not load task data from file '%s': Exception raised: %s" % (fullname, str(e)))
            _logger.info("The task file '%s' will be removed from the task storage" % fullname)
            osutil.unlink_nothrow(fullname)
            return None
        return task_item
        

    def getTask(self,task_id):
        task_id_str = str(task_id)
        tasksDirectory = self.persistent_storage_dir
        if os.path.isdir(tasksDirectory):
            fullname = os.path.join(tasksDirectory, task_id_str)
            task_item = None
            if os.path.isfile(fullname):
                task_item = self.__loadTask(fullname)
            return task_item

    def getTasks(self, types, owner_guid, topobject_id, topobject_type):
        tasksDirectory = self.persistent_storage_dir
        task_list = []
        if os.path.isdir(tasksDirectory):
            names = os.listdir(tasksDirectory)
            for name in names:
                fullname = os.path.join(tasksDirectory, name)
                if os.path.isfile(fullname):
                    # name is a task number
                    task_item = self.__loadTask(fullname)
                    if task_item is not None:
                        if task_item.get_tasktype() in types:
                            # FS specifies that owner_guid, topobject_id, topobject_type are satisfied with followings:
                            # if some object is None, the rest objects in list are None too
                            if isinstance(task_item,BaseDictionaryTask):
                                if owner_guid:
                                    if task_item.has_key('owner_guid') and (not owner_guid in ["any", task_item.get('owner_guid')]):
                                        continue
                                    elif topobject_id:
                                        if task_item.has_key('topobject_id') and (not topobject_id in ["any", task_item.get('topobject_id')]):
                                            continue
                                        elif topobject_type:
                                            if task_item.has_key('topobject_type') and (not topobject_type == task_item.get('topobject_type')):
                                                continue
                            task_list.append(task_item)
        # sort task list by creation_date
        temp_array = [(-item.get_creation_date(), index, item) for index, item in enumerate(task_list)]
        temp_array.sort()
        return [item[-1] for item in temp_array]

    def getUniqueTaskPid(self, pid):
        tasksDirectory = self.persistent_storage_dir
        taskPids = []
        if os.path.isdir(tasksDirectory):
            taskPids = os.listdir(tasksDirectory)
        uniqueTaskPid = pid
        if uniqueTaskPid < 0 or str(uniqueTaskPid) in taskPids:
            uniqueTaskPid = 2147483647
            while str(uniqueTaskPid) in taskPids:
                uniqueTaskPid = uniqueTaskPid - 1

        return uniqueTaskPid

    def saveTask(self, task):
        self.__make_persistent_storage()
        dumper = FileDumpMarshaller(os.path.join(self.persistent_storage_dir,str(task.get_task_id())))
        task.dump(dumper)
        dumper.close()

    def removeTask(self, task):
        task_item_file = os.path.join(self.persistent_storage_dir,str(task.get_task_id()))
        if os.path.isfile(task_item_file):
            osutil.unlink_nothrow(task_item_file)


class FinishedTaskStatusCreator:
    def __init__(self, task):
        self.__task = task

    def response(self):
        task_status = Finished( status = 'success')
        return TaskStatus ( finished = task_status )


class FinishedBackupTaskStatusCreator(FinishedTaskStatusCreator):
    def __init__(self, task):
        if not isinstance(task, BackupTask):
            raise InvalidTaskOperation, 'FinishedBackupTaskStatusCreator does not bind with BackupTask instance' % task.__class__.__name__
        self.__task = task
        fullname = self.__task.get('fullname')
        if fullname == None or fullname == '':
            dump_file_name_file_name = os.path.join(self.__task.get('session_path'),'dump-name')
            if os.path.isfile(dump_file_name_file_name):
                dump_file_name_file = open(dump_file_name_file_name,"rt")
                # pmm-ras for local storage on unix returns file name that starts with '/'
                # we should remove it from filename
                dump_file_name = dump_file_name_file.readline().rstrip('\n').lstrip('/\\')
                dump_file_name_file.close()
                if dump_file_name != '':
                    # dump file name here could be
                    formatter = pmm_dump_formatter.DumpsStorageCredentialsFormatter(self.__task.get('dumps_storage_credentials'))
                    storage = formatter.getDumpStorageNoLogin()
                    if formatter.get_storage_type() == 'foreign-ftp':
                        # pmm-ras could return mixed separators in the name of exported file for ftp storage type
                        dump_file_name = dump_file_name.replace('\\','/')
                    file_name = dump_file_name.replace(storage,'')
                    self.__task.set('fullname',file_name)
                    self.__task.set('name',file_name)
                    getPMMTaskManager().updateTask(self.__task)
        err_file_name = os.path.join(self.__task.get('session_path'),'.stderr')
        err_string = None
        if os.path.isfile(err_file_name):
            err = open(err_file_name,"r")
            err_string = err.read()
            err.close()
            shutil.copy2(err_file_name,os.path.join(self.__task.get('session_path'),'stderr'))
            osutil.unlink_nothrow(err_file_name)
        
        if err_string and err_string != '':
            stderr_string = "\n== STDERR ====================\n" + err_string + "\n==============================\n"
            log_file_name = self.__task.get('log_filename')
            if os.path.isfile(log_file_name):
                log_file = open(log_file_name,"a")
                log_file.write(stderr_string)
                log_file.close()

    def response(self):
        task_status = None
        pid = str(self.__task.get_task_id())
        status_file_path = os.path.join(self.__task.get('session_path'),'migration.result')
        _logger.debug("Get task (%s) status from migration.result..." % pid)
        _logger.debug("Session path (%s)" % self.__task.get('session_path'))
        if os.path.isfile(status_file_path):
            #load migration result from file
            try:
                getter = pmm_dump_access_service.XmlElementTextGetter('execution-result')
                status_getter = pmm_dump_access_service.XmlElementAttrGetter('execution-result','status')
                xml_processor = pmm_dump_access_service.InfoXMLProcessor( status_file_path, pmm_dump_access_service.XmlHandler([getter,status_getter]))
                xml_processor.process()
                status_attribute = status_getter.get_attribute_value()
                # task working process (backup or deployer) writes only 'error' or 'success' as status of finished task
                # if status task is not 'error' we should suppose it is 'success'
                if status_attribute != 'error':
                    status_attribute = 'success'
                status_value = getter.get_element_inner_text()
                # if execution result has any context then reduce status to 'warning'
                if status_attribute == 'success' and status_value.strip() != '':
                    status_attribute = 'warnings'
                task_status = Finished( status = status_attribute, log_location = self.__task.get('migration_result_filename'))
            except:
                _logger.debug("Execution result file %s exists but has wrong format. The task %s will be marked as finished with success." % (status_file_path, pid))
                task_status = Finished( status = 'success', log_location = self.__task.get('migration_result_filename'))
        else:
            _logger.debug("Execution result file %s isn't found. The task %s will be marked as finished with error." % (status_file_path, pid))
            task_status = Finished( status = 'error', log_location = self.__task.get('migration_result_filename'))
        
        return TaskStatus ( finished = task_status )


class FinishedRestoreTaskStatusCreator(FinishedTaskStatusCreator):
    def __init__(self, task):
        if not isinstance(task, RestoreTask):
            raise InvalidTaskOperation, 'FinishedRestoreTaskStatusCreator does not bind with RestoreTask instance' % task.__class__.__name__
        self.__task = task
        fullname = self.__task.get('fullname')
        if fullname == None or fullname == '':
            dump_file_name_file_name = os.path.join(self.__task.get('session_path'),'dump-name')
            if os.path.isfile(dump_file_name_file_name):
                dump_file_name_file = open(dump_file_name_file_name,"rt")
                # pmm-ras for local storage on unix returns file name that starts with '/'
                # we should remove it from filename
                dump_file_name = dump_file_name_file.readline().rstrip('\n').lstrip('/\\')
                dump_file_name_file.close()
                if dump_file_name != '':
                    # dump file name here could be
                    formatter = pmm_dump_formatter.DumpsStorageCredentialsFormatter(self.__task.get('dumps_storage_credentials'))
                    storage = formatter.getDumpStorageNoLogin()
                    if formatter.get_storage_type() == 'foreign-ftp':
                        # pmm-ras could return mixed separators in the name of exported file for ftp storage type
                        dump_file_name = dump_file_name.replace('\\','/')
                    file_name = dump_file_name.replace(storage,'')
                    self.__task.set('fullname',file_name)
                    self.__task.set('name',file_name)
                    getPMMTaskManager().updateTask(self.__task)
        err_file_name = os.path.join(self.__task.get('session_path'),'.stderr')
        err_string = None
        if os.path.isfile(err_file_name):
            err = open(err_file_name,"r")
            err_string = err.read()
            err.close()
            shutil.copy2(err_file_name,os.path.join(self.__task.get('session_path'),'stderr'))
            osutil.unlink_nothrow(err_file_name)
        
        if err_string and err_string != '':
            stderr_string = "\n== STDERR ====================\n" + err_string + "\n==============================\n"
            log_file_name = self.__task.get('log_filename')
            if os.path.isfile(log_file_name):
                log_file = open(log_file_name,"a")
                log_file.write(stderr_string)
                log_file.close()

    def response(self):
        task_status = None
        pid = str(self.__task.get_task_id())
        status_file_path = os.path.join(self.__task.get('session_path'),'migration.result')
        _logger.debug("Get task (%s) status from migration.result..." % pid)
        _logger.debug("Session path (%s)" % self.__task.get('session_path'))
        if os.path.isfile(status_file_path):
            #load migration result from file
            try:
                getter = pmm_dump_access_service.XmlElementTextGetter('execution-result')
                status_getter = pmm_dump_access_service.XmlElementAttrGetter('execution-result','status')
                xml_processor = pmm_dump_access_service.InfoXMLProcessor( status_file_path, pmm_dump_access_service.XmlHandler([getter,status_getter]))
                xml_processor.process()
                status_attribute = status_getter.get_attribute_value()
                # task working process (backup or deployer) writes only 'error' or 'success' as status of finished task
                # if status task is not 'error' we should suppose it is 'success'
                if status_attribute != 'error':
                    status_attribute = 'success'
                status_value = getter.get_element_inner_text()
                # if execution result has any context then reduce status to 'warning'
                if status_attribute == 'success' and status_value.strip() != '':
                    status_attribute = 'warnings'
                task_status = Finished( status = status_attribute, log_location = self.__task.get('migration_result_filename'))
            except:
                _logger.debug("Execution result file %s exists but has wrong format. The task %s will be marked as finished with success." % (status_file_path, pid))
                task_status = Finished( status = 'success', log_location = self.__task.get('migration_result_filename'))
        else:
            _logger.debug("Execution result file %s isn't found. The task %s will be marked as finished with error." % (status_file_path, pid))
            task_status = Finished( status = 'error', log_location = self.__task.get('migration_result_filename'))
        
        return TaskStatus ( finished = task_status )


class StoppedTaskStatusCreator:
    def __init__(self, task):
        self.__task = task

    def response(self):
        return TaskStatus ( stopped = Stopped() )


class BaseDictionaryTaskLogCreator:
    def __init__(self, task):
        self.__task = task
        if not isinstance(task, BaseDictionaryTask):
            raise InvalidTaskOperation, "BaseDictionaryTaskLogCreator does not bind with BaseDictionaryTask instance %s" % task.__class__.__name__
        if not task.has_key('log_filename'):
            raise InvalidTaskOperation, "BaseDictionaryTaskLogCreator error: BaseDictionaryTask instance (%s) does not have 'log_filename' dictionary key" % task.__class__.__name__
        self.__task_log = task.get('log_filename')

    def response(self):
        log_string = "Log file \'" + self.__task_log + "\' does not exist"
        
        if os.path.isfile(self.__task_log):
            log = open(self.__task_log,"rt")
            log_string = log.read().decode('latin-1')
            log.close()
        
        return TaskLog ( log_string.encode('utf-8') )


class RunningTaskStatusCreator:
    def __init__(self, task):
        self.__task = task

    def response(self):
        task_status = Working( starting = Starting() )
        return TaskStatus ( working = task_status )


class RunningBackupTaskStatusCreator(RunningTaskStatusCreator):
    def __init__(self, task):
        if not isinstance(task, BackupTask):
            raise InvalidTaskOperation, 'RunningBackupTaskStatusCreator does not bind with BackupTask instance (%s)' % task.__class__.__name__
        self.__task = task

    def response(self):
        task_status = None
        status_file_path = os.path.join(self.__task.get('session_path'),'dump-status.xml')
        if os.path.isfile(status_file_path):
            #load dumping status from file
            status_file = open(status_file_path,"rt")
            status_file_content = status_file.read()
            status_file.close()
            dumping_status = WorkingProgress.factory()
            dumping_status.build(minidom.parseString(status_file_content).childNodes[0])
            task_status = Working( dumping = dumping_status )
        else:
            task_status = Working( starting = Starting() )
        
        return TaskStatus ( working = task_status )


class RunningRestoreTaskStatusCreator(RunningTaskStatusCreator):
    def __init__(self, task):
        if not isinstance(task, RestoreTask):
            raise InvalidTaskOperation, 'RunningRestoreTaskStatusCreator does not bind with RestoreTask instance (%s)' % task.__class__.__name__
        self.__task = task

    def response(self):
        task_status = None
        status_file_path = os.path.join(self.__task.get('session_path'),'migration.status')
        if os.path.isfile(status_file_path):
            #load deploy status from file
            status_file = open(status_file_path,"rt")
            status_file_content = status_file.read()
            status_file.close()
            deploy_status = WorkingProgress.factory()
            deploy_status.build(minidom.parseString(status_file_content).childNodes[0])
            task_status = Working( deploy = deploy_status )
        else:
            task_status = Working( starting = Starting() )
        
        return TaskStatus ( working = task_status )


pmmTaskManager = None


def getPMMTaskManager():
    global pmmTaskManager
    if not pmmTaskManager:
        pmmTaskManager = PMMTaskManager()
    return pmmTaskManager


class PMMTaskManager(object):
    def __init__(self):
        self.persistent_storage = PMMTaskPersistentStorage()
        self.stateMutex = RLock()

    def realGetTaskList(self, types, owner_guid = None, topobject_id = None, topobject_type = None):
        task_list = self.persistent_storage.getTasks(types, owner_guid, topobject_id, topobject_type)
        for task in task_list:
            self.operatorGetTaskStatus(task)
        return task_list

    def realGetTask(self, task_id):
        task = self.persistent_storage.getTask(task_id)
        self.operatorGetTaskStatus(task)
        return task

    def __operatorGetTaskLog(self, task):
        return task.get_log()

    def operatorGetTaskStatus(self, task):
        return task.get_status()

    def __operatorGetTask(self, task):
        task_list = TaskList.factory()
        dumps_storage_credentials_value = None
        owner_guid_value = None
        owner_type_value = None
        fullname_value = None
        name_value = None
        backup_profile_name_value = None
        mailto_value = None
        additional_info_value = None
        if isinstance(task,BaseDictionaryTask):
            if task.has_key('dumps_storage_credentials'):
                dumps_storage_credentials_value = task.get('dumps_storage_credentials')
            if task.has_key('owner_guid'):
                owner_guid_value = task.get('owner_guid')
            if task.has_key('owner_type'):
                owner_type_value = task.get('owner_type')
            if task.has_key('fullname'):
                fullname_value = task.get('fullname')
            if task.has_key('name'):
                name_value = task.get('name')
            if task.has_key('backup_profile_name'):
                backup_profile_name_value = task.get('backup_profile_name')
            if task.has_key('mailto'):
                mailto_value = task.get('mailto')
            if task.has_key('additional_info'):
                additional_info_value = task.get('additional_info')
        if dumps_storage_credentials_value:
            dumps_storage_credentials_formatter = pmm_dump_formatter.DumpsStorageCredentialsFormatter(dumps_storage_credentials_value)
            task_object = TaskElement.factory(task_id = task.get_task_id(), owner_guid = owner_guid_value, owner_type = owner_type_value, fullname = fullname_value, creation_date = task.get_creation_date_formatted(), name = name_value, backup_profile_name = backup_profile_name_value, mail_to = mailto_value, task_status = self.operatorGetTaskStatus(task), dumps_storage_credentials = dumps_storage_credentials_formatter.buildXml(), additional_info = additional_info_value)
        else :
            task_object = TaskElement.factory(task_id = task.get_task_id(), owner_guid = owner_guid_value, owner_type = owner_type_value, fullname = fullname_value, creation_date = task.get_creation_date_formatted(), name = name_value, backup_profile_name = backup_profile_name_value, mail_to = mailto_value, task_status = self.operatorGetTaskStatus(task), additional_info = additional_info_value)
        
        task_list.add_task(task_object)
        return task_list

    def __operatorStopTask(self, task):
        if not task.get_stopped():
            task.stop()
            self.persistent_storage.saveTask(task)
        return 1

    def __operatorRemoveTask(self,task):
        self.__operatorStopTask(task)
        task.remove()
        self.persistent_storage.removeTask(task)
        return 1

    def __taskFinder(self, task_id, operator):
        task = self.persistent_storage.getTask(task_id)
        if task:
            self.operatorGetTaskStatus(task)
            return operator(task)

    def updateTask(self, task):
        _logger.debug("Acquired session mutex: " + currentThread().getName())
        self.stateMutex.acquire()
        try:
            self.persistent_storage.saveTask(task)
        finally:
            self.stateMutex.release()
            _logger.debug("Released session mutex: " + currentThread().getName())

    def addTask(self, task):
        _logger.debug("Acquired session mutex: " + currentThread().getName())
        self.stateMutex.acquire()
        pid = task.get_os_pid()
        created_stopped = pid < 0
        try:
            if created_stopped:
                task.stop()
            task.set_task_id(self.persistent_storage.getUniqueTaskPid(pid))
            if not created_stopped:
                # start pmmcli_daemon
                pmmcli_daemon_service.PMMCliDaemon().start()
            self.persistent_storage.saveTask(task)
        
        finally:
            self.stateMutex.release()
            _logger.debug("Released session mutex: " + currentThread().getName())


    def stopTask(self, task_id):
        _logger.debug("Acquired session mutex: " + currentThread().getName())
        self.stateMutex.acquire()
        try:
            result = self.__taskFinder(task_id,self.__operatorStopTask)
        finally:
            self.stateMutex.release()
            _logger.debug("Released session mutex: " + currentThread().getName())
        return result

    def removeTask(self, task_id):
        _logger.debug("Acquired session mutex: " + currentThread().getName())
        self.stateMutex.acquire()
        try:
            result = self.__taskFinder(task_id,self.__operatorRemoveTask)
        finally:
            self.stateMutex.release()
            _logger.debug("Released session mutex: " + currentThread().getName())
        return result

    # the 'getXxxx' methods return XML-element-class instances autogenerated from 'pmm_api_xml_protocols.xsd'
    def getTaskList(self, types, owner_guid, topobject_id, topobject_type):
        task_list = []
        _logger.debug("Acquired session mutex: " + currentThread().getName())
        self.stateMutex.acquire()
        try:
            task_list = self.realGetTaskList(types, owner_guid, topobject_id, topobject_type)
        finally:
            self.stateMutex.release()
            _logger.debug("Released session mutex: " + currentThread().getName())
        task_object_list = TaskList.factory()
        for task_item in task_list:
            task_object = None
            dumps_storage_credentials_value = None
            owner_guid_value = None
            owner_type_value = None
            fullname_value = None
            name_value = None
            backup_profile_name_value = None
            mailto_value = None
            additional_info_value = None
            if isinstance(task_item,BaseDictionaryTask):
                if task_item.has_key('dumps_storage_credentials'):
                    dumps_storage_credentials_value = task_item.get('dumps_storage_credentials')
                if task_item.has_key('owner_guid'):
                    owner_guid_value = task_item.get('owner_guid')
                if task_item.has_key('owner_type'):
                    owner_type_value = task_item.get('owner_type')
                if task_item.has_key('fullname'):
                    fullname_value = task_item.get('fullname')
                if task_item.has_key('name'):
                    name_value = task_item.get('name')
                if task_item.has_key('backup_profile_name'):
                    backup_profile_name_value = task_item.get('backup_profile_name')
                if task_item.has_key('mailto'):
                    mailto_value = task_item.get('mailto')
                if task_item.has_key('additional_info'):
                    additional_info_value = task_item.get('additional_info')
            if dumps_storage_credentials_value:
                dumps_storage_credentials_formatter = pmm_dump_formatter.DumpsStorageCredentialsFormatter(dumps_storage_credentials_value)
                task_object = TaskElement.factory(task_id = task_item.get_task_id(), name = name_value, mail_to = mailto_value, owner_guid = owner_guid_value, owner_type = owner_type_value, backup_profile_name = backup_profile_name_value, fullname = fullname_value, creation_date = task_item.get_creation_date_formatted(), task_status = self.operatorGetTaskStatus(task_item), dumps_storage_credentials = dumps_storage_credentials_formatter.buildXml(), additional_info = additional_info_value)
            else :
                task_object = TaskElement.factory(task_id = task_item.get_task_id(), name = name_value, mail_to = mailto_value, owner_guid = owner_guid_value, owner_type = owner_type_value, backup_profile_name = backup_profile_name_value, fullname = fullname_value, creation_date = task_item.get_creation_date_formatted(), task_status = self.operatorGetTaskStatus(task_item), additional_info = additional_info_value)
            task_object_list.add_task(task_object)
        return task_object_list

    def getTask(self, task_id):
        _logger.debug("Acquired session mutex: " + currentThread().getName())
        self.stateMutex.acquire()
        try:
            result = self.__taskFinder(task_id,self.__operatorGetTask)
        finally:
            self.stateMutex.release()
            _logger.debug("Released session mutex: " + currentThread().getName())
        return result

    def getTaskStatus(self, task_id):
        _logger.debug("Acquired session mutex: " + currentThread().getName())
        self.stateMutex.acquire()
        try:
            result = self.__taskFinder(task_id,self.operatorGetTaskStatus)
        finally:
            self.stateMutex.release()
            _logger.debug("Released session mutex: " + currentThread().getName())
        return result

    def getTaskLog(self, task_id):
        _logger.debug("Acquired session mutex: " + currentThread().getName())
        self.stateMutex.acquire()
        try:
            result = self.__taskFinder(task_id,self.__operatorGetTaskLog)
        finally:
            self.stateMutex.release()
            _logger.debug("Released session mutex: " + currentThread().getName())
        return result

