import commands
import getopt
import codecs
import os
import osutil
import sys
import datetime 
import xml.dom.minidom
import shutil
import random

import backup
import backup_walker
import dump_walker
import validator
import config

class ValidatorErrorHandler:
    def __init__(self):
        self.errors = []
    def handler(self, msg, data):
        self.errors.append(msg)
    def __unicode__(self):
        return u"\n".join(self.errors)

class ExtractDumpVisitor(object):
    def __init__(self, outDir, verbosity, validate):
        self.outDir = outDir
        self.verbose = verbosity
        self.validate = validate
        self.objects = {}
        self.infoObjects = []
        self.cids = {}
        self.date = datetime.datetime.now();
        self.pathElements = {}
        self.infoXmlSizes = {}
        self.contentSizes = {}

    def log(self, message):
        if self.verbose:
            print message;

    def logError(self, message):
        print 'Error: ' + message;

    def visit(self, info, x):
        for cid, fileName in info.getCids().items():
            cidId = info.getCidId(cid)
            self.cids[cidId] = info

        self.infoObjects.append((x, info))

    def writeMetadata(self, sign):
        
        rootInfo = None
        
        for node, info in self.infoObjects:
            if not info.needSave():
                continue
            dir = info.objectFullDir()
            targetDir = os.path.join(self.outDir, dir.encode("utf-8"))
            self.log(">> %s" % targetDir)
            if not os.path.isdir(targetDir):
                os.makedirs(targetDir)
            backupName = self.getBackupName(info, None) + ".xml"
            savePath = os.path.join(targetDir, backupName)
            size = self.saveNode(node, info, savePath, sign)
            self.infoXmlSizes[os.path.join(dir, backupName)] = size
            
            if self.validate:
                self.validateXml(savePath, sign)
            
            if info.getType() == 'server':
                rootInfo = info
            elif info.getType() == 'client':
                if rootInfo == None or rootInfo.getType() != 'server':
                     rootInfo = info
            elif info.getType() == 'domain':
                if rootInfo == None or (rootInfo.getType() != 'server' and rootInfo.getType() != 'client'):
                     rootInfo = info
        
        for node, info in self.infoObjects:
            if not info.needSave():
                continue
            dir = info.objectFullDir()
            
            targetDir = os.path.join(self.outDir, dir.encode("utf-8"))
            if not os.path.isdir(targetDir):
                os.makedirs(targetDir)
            targetDir = os.path.join(targetDir, '.discovered')
            if not os.path.isdir(targetDir):
                os.makedirs(targetDir)
            targetDir = os.path.join(targetDir, self.getBackupName(info, None))
            if not os.path.isdir(targetDir):
                os.makedirs(targetDir)
            
            ownerPath = os.path.join(targetDir, 'owner_00000000-0000-0000-0000-000000000000')
            self.createFile(ownerPath)            
            ownerTypePath = os.path.join(targetDir, 'ownertype_server')
            self.createFile(ownerTypePath)
            if info.getGuid() != None and info.getGuid() != '':
                guidPath = os.path.join(targetDir, 'GUID_' + info.getGuid())
                self.createFile(guidPath)
            objectIdPath = os.path.join(targetDir, 'objectid_' + info.getGuidName())
            self.createFile(objectIdPath)
            sizePath = os.path.join(targetDir, 'size_' + self.collectSize(dir).__str__())
            self.createFile(sizePath)
            
            if info.getGuid() != None and info.getGuid() != '':
                self.writeTopDiscovered(info.getGuidName(), info.getGuid())
                
        if rootInfo == None:
            return ''
        return self.getBackupName(rootInfo, None) + ".xml"

    def writeTopDiscovered(self, name, guid):
        targetDir = os.path.join(self.outDir, '.discovered')
        if not os.path.isdir(targetDir):
            os.makedirs(targetDir)

        itemPath = os.path.join(targetDir, guid + '_' + name)
        self.createFile(itemPath)
        

    def collectSize(self, dir):
        total = 0
        
        for path, size in self.infoXmlSizes.iteritems():
            if path.startswith(dir):
                total += size

        for path, size in self.contentSizes.iteritems():
            if path.startswith(dir):
                total += size
        
        return total

    def createFile(self, path):
        if not os.path.isfile(path):
            try:
                file = open(path,"w")
                file.close()
            except Exception, e:
                self.logError('unable to create file \'' + path + '\': ' + e.__str__())

    def validateXml(self, path, sign):
        self.log("Validating '" + path + "' ...")
        import libxml2

        tmpPath = path + '.tmp'
        if sign:
            self.callBackupSign(path, tmpPath, 'verify')
            path = tmpPath

        ctxt_parser = libxml2.schemaNewParserCtxt(config.getPMMDir() + "plesk.xsd")
        ctxt_schema = ctxt_parser.schemaParse()
        ctxt_valid = ctxt_schema.schemaNewValidCtxt()
        
        erh = ValidatorErrorHandler()

        ctxt_valid.setValidityErrorHandler(erh.handler, erh.handler)

        file = open(path, 'r')
        xmlstr = file.read()
        file.close()

        doc = libxml2.parseDoc(xmlstr)
        ret = doc.schemaValidateDoc(ctxt_valid)
        doc.freeDoc()
        del ctxt_valid
        del doc

        if ret != 0:
            print "File '" + path.encode("utf-8") + "' validation failed:\n" + unicode(erh)
        else:
            self.log("File succesfully validated.")

        if sign:
            os.remove(tmpPath)

    def saveNode(self, node, info, savePath, sign):
        self.processSavingNode(node, info)
        
        tmpPath = savePath + '.tmp'
        if sign:
            f = file(tmpPath, "wb")
        else:
            f = file(savePath, "wb")
        
        f.write("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>")
        if node.nodeName != "migration-dump":
            f.write("<migration-dump agent-name=\"pre9-backup-convert\" dump-version=\"9.0\">")
        #f.write(node.toprettyxml().encode('utf-8'))
        node.writexml(codecs.lookup("UTF-8")[3](f))
        if node.nodeName != "migration-dump":
            f.write("</migration-dump>")
        f.close()
        
        if sign:
            self.callBackupSign(tmpPath, savePath, 'sign')
            os.remove(tmpPath)
        
        return os.path.getsize(savePath)
        
    def callBackupSign(self, fromPath, toPath, command):
        cmd = config.getBackupSign() + ' ' + command + ' ' + fromPath + ' >' + toPath
        self.log("exec: " + cmd)
        os.system(cmd)

    
    def processSavingNode(self, node, info):
        if self.pathElements.has_key(node):
            nodeObj = self.pathElements[node]
            pathDiff = info.getPathDiff(nodeObj)
            if pathDiff == None or pathDiff == "":
                for i in range(node.attributes.length):
                    if node.attributes.item(i).name == "path":
                        node.attributes.remove(i)
            else:
                node.setAttribute("path", pathDiff)
                for contentFile in dump_walker.childElementsName(node, 'content-file'):
                    if not contentFile.hasAttribute('size'):
                        contentPath = os.path.join(nodeObj.objectFullDir(), dump_walker.textContent(contentFile))
                        filePath = os.path.join(self.outDir, contentPath)                        
                        try:
                            size = os.path.getsize(filePath)
                            contentFile.setAttribute('size', size.__str__())
                            self.contentSizes[contentPath] = size
                        except Exception, e:
                            self.logError('unable to get size of file: ' + e.__str__())
            
        for n in node.childNodes:
            if n.nodeType == xml.dom.minidom.Node.ELEMENT_NODE:
                self.processSavingNode(n, info)

    def unpackCid(self, cid, bundle):
        if cid not in self.cids:
            self.log("IGN cid: " + cid)
            return
        info = self.cids[cid]
        dir = info.objectFullDir().encode("utf-8")
        specialContent = ["key", "webuser_home"]
        if info.getCidType(cid) not in specialContent:
            fileName = (self.getBackupName(info, info.getCidType(cid)) + ".tgz").encode("utf-8")
        else:
            fileName = (self.getBackupName(info, info.getCidType(cid)) + "_" + cid + ".tgz").encode("utf-8")

        outFile = os.path.join(self.outDir, dir, fileName)
        
        targetDir = os.path.dirname(outFile)
        if not os.path.isdir(targetDir):
            os.makedirs(targetDir)

        tmpFile = os.path.join(self.outDir, dir, cid)
        bundle.process(osutil.createToFileSaver(tmpFile))
        if info.getCidType(cid) == 'sqldump':
            cmd = "tar -f " + outFile + " -c -z -C " + os.path.join(self.outDir, dir) + " " + cid
            self.log("tar exec: " + cmd)
            os.system(cmd)
        else:
            cmd = "gzip -c " + tmpFile + ">" + outFile
            self.log("gzip exec: " + cmd)
            os.system(cmd)
        
        os.remove(tmpFile)
        
        info.setCidFileName(cid, fileName)
    
    # path factory members
        
    def getBackupName(self, infoObject, type):
        retStr = "converted_" 
        
        name = infoObject.getName()
        if name != None and name != "":
            retStr += name + "_"
        
        if type != None:
            retStr += dump_walker.normalizeString(type, None, None) + "_"
        else:
            retStr += "info_"
        
        retStr += self.date.strftime("%y%m%d%H%M")
        
        return retStr.encode("utf-8")

    def registerElementObject(self, x, object):
        self.pathElements[x] = object

class ExtractBackupVisitor(object):
    def __init__(self, outDir, verbosity, validate):
        self.verbose = verbosity
        self.validate = validate
        self.extractor = ExtractDumpVisitor(outDir, self.verbose, self.validate)

    def visitMetadata(self, metadata):
        def processMetadata(fh):
            dump_walker.walkDump(self.extractor, fh, self.extractor)
        metadata.process(processMetadata)

    def visitBundle(self, contentId, bundle):
        self.extractor.unpackCid(contentId, bundle)

    def saveMetadata(self, metadata, sign):
        return self.extractor.writeMetadata(sign)
        

class ConvertCommand(commands.HelpProvider):
    USAGE_STRING = """convert 
                     [-d <output-directory>|--directory=<output-directory>] <backup-file>"""
    SHORT_HELP_STRING = "Converts backups of Plesk version < 9.0"
    HELP_STRING = """Converts backups of Plesk version < 9.0.

-d|--directory option explicitly sets the directory for extracting the backup
content. By default directory named after backup file will be created in current
directory.
"""

    def __init__(self, verbosity):
        random.seed()
        self.verbosity = verbosity
        self.tempFiles = []

    def parse(self, args):
        if len(args) == 0:
            raise RuntimeError("No backup file specified")
        self.backupFile = args.pop().encode("utf-8")

        recursive = True
        self.outDir = (os.path.basename(self.backupFile) + ".converted/").encode("utf-8")

        self.validate = False

        opts, args = getopt.gnu_getopt(args, "d:l", ["directory=", "validate"])
        for name, value in opts:
            if name in ("-d", "--directory"):
                self.outDir = value.encode("utf-8")
            elif name in ("-l", "--validate"):
                self.validate = True

        if not self.outDir.endswith("/"):
            self.outDir += "/"

    def act(self):
        backupFile = backup.backupFileFactory('file', self.backupFile)
        if not backupFile.exists():
            raise RuntimeError("Backup file '" + self.backupFile + "' does not exist")

        if os.path.exists(self.outDir):
            if not os.path.isdir(self.outDir):
                raise RuntimeError(self.outDir + ": Not a directory")
        else:
            os.mkdir(self.outDir)
        
        b = backup.Backup('file', self.backupFile)
        ebv = ExtractBackupVisitor(self.outDir, self.verbosity, self.validate)
        createdPath = backup_walker.walkBackup(b, ebv)
        
        print 'Created dump:'
        print createdPath

commands.register("convert", ConvertCommand, 1)

# Local Variables:
# mode: python
# indent-tabs-mode: nil
# tab-width: 4
# End:
