from threading import BoundedSemaphore, currentThread
from encoder import EncoderFile
import os, cStringIO, logging
import panel_detection
from backend_interface import *
import pmm_config

try: import subprocess
except ImportError: import compat.subprocess as subprocess

_logger = logging.getLogger("pmm.supervisor.session")

class Session(object):
    def __init__(self, startState, sessionDirectory):
        self.connectivityInfo = None
        self.apiConnectivityInfo = None
        self.apiRemoteAgentDir = None
        self.sessionDirectory = sessionDirectory
        self.agentParameters = None
        self.sessionParameters = None
        self.selectedObjects = None
        self.importMode = False
        self.extractor_pid = None

        self.stateMutex = BoundedSemaphore()
        self.setState(startState)

    def __setState(self, newState):
        _logger.debug("-> " + newState.__class__.__name__)
        self.state = newState
        self.state.setSession(self)
        self.state.do()

    def processRequest(self, action):
        _logger.debug("Acquired session mutex: " + currentThread().getName())
        self.stateMutex.acquire()
        try:
            _logger.debug("Processing request")
            (newState, response) = self.state.processRequest(action)
            response.setAction_internal_name(self.state.__class__.__name__)
            if newState:
                self.__setState(newState)
        finally:
            _logger.debug("Released session mutex: " + currentThread().getName())
            self.stateMutex.release()
        return response

    def setState(self, newState):
        _logger.debug("Acquired session mutex: " + currentThread().getName())
        self.stateMutex.acquire()
        try:
            self.__setState(newState)
        finally:
            _logger.debug("Released session mutex: " + currentThread().getName())
            self.stateMutex.release()

    def options(self, cmdline, *options):
        for o in options:
            self.__getattribute__("_Session__get" + o + "Options")(cmdline)

    def __getConnectivityOptions(self, cmdline):
        ci = self.connectivityInfo

        cmdline.arg("--host=" + ci.getHost())
        if ci.getPort():
            cmdline.arg("--port=" + ci.getPort())

        if ci.getSudo():
            cmdline.arg("--login=" + ci.getSudo().getUser())
            cmdline.arg("--sudo-password")
            cmdline.var("PSA_SUDO_PASSWORD", ci.getPassword())
            cmdline.var("PSA_PASSWORD", ci.getSudo().getPassword())
        else:
            cmdline.arg("--login=root")
            cmdline.var("PSA_PASSWORD", ci.getPassword())

    def __getSessionOptions(self, cmdline):
        cmdline.arg("--session-path=" + self.sessionDirectory)

    def __getAgentOptions(self, cmdline):
        cmdline.arg("--agent-name=" + self.agentParameters.getAgent_name())

    def __getAgentDirOptions(self, cmdline):
        cmdline.arg("--agent-dir=" + self.agentParameters.getRemote_agent_dir())

    def __getDumpOptions(self, cmdline):
        if not self.agentParameters.getCompress_content():
            cmdline.arg("--no-compress")

    def __getGetContentOptions(self, cmdline):
        if not self.agentParameters.getCheck_disk_space():
            cmdline.arg("--no-check-space")

    def __getDeployOptions(self, cmdline):
        if self.sessionParameters.getDomain_template():
            cmdline.arg("--domain-template=" + self.sessionParameters.getDomain_template())
        if self.sessionParameters.getSend_notifications():
            cmdline.arg("--notify")

    def __getDumpMigratingObjectsOptions(self, cmdline):
        so = self.selectedObjects

        if so.getMigrate_domains():
            domains = []
            for domainList in so.getMigrate_domains():
                domains.extend(domainList.getDomain())

            domainNames = [d.getName() for d in domains]
            cmdline.arg("--dump-domains=-").addstdin((",".join(domainNames)) + "\n")
        else:
            clients = ",".join([c.getName() for c in so.getMigrate_clients().getClient()])
            cmdline.arg("--dump-accounts=-").addstdin(clients + "\n")

    def getIpMap(self, cmdline):
        resp_str = cStringIO.StringIO()
        resp_encoded = EncoderFile(resp_str, "utf-8")
        self.selectedObjects.export(resp_encoded, 0, name_ = 'selected-objects')
        cmdline.addstdin(resp_str.getvalue())

    def detectPanel(self):
        try:
            panel = panel_detection.detect(os.path.join(self.sessionDirectory, "dump.xml"))
            if panel == "":
                return (None, "'agent-name' attribute is not found in dump file")
            return (panel, None)
        except Exception, e:
            return (None, str(e))

    def getConvertedDumpFileName(self):
        return os.path.join(self.sessionDirectory, "dump-plesk.xml")

    def getResultsFileName(self):
        return os.path.join(self.sessionDirectory, "migration.result")

    def getDomainsCount(self):
        so = self.selectedObjects

        if so.getMigrate_domains():
            domainNum = 0
            for domainList in so.getMigrate_domains():
                domainNum += len(domainList.getDomain())
            return domainNum
        else:
            domainsNum = 0
            for client in so.getMigrate_clients().getClient():
                domainsNum += len(client.getDomain())
            return domainsNum

    def getClientsCount(self):
        so = self.selectedObjects

        if so.getMigrate_domains():
            return 0
        else:
            return len(so.getMigrate_clients().getClient())


    def getDumpFileName(self):
        return os.path.join(self.sessionDirectory, "dump.xml")

    def getPleskDumpFileName(self):
        return os.path.join(self.sessionDirectory, "dump-plesk.xml")

    def getArchivesDir(self):
        return os.path.join(self.sessionDirectory, "archives")

    def processLogs(self):
        destDirectory = os.path.join(pmm_config.pmm_logs_directory(),os.path.basename(self.sessionDirectory))
        if not os.path.exists(destDirectory):
          os.makedirs(destDirectory)
        res = os.path.join(destDirectory,"migration-logs.tar")  
        os.umask(007)
        retcode = subprocess.call("tar " + "cf " + res + " " + self.sessionDirectory + " --exclude archives &>/dev/null", shell=True)
        return res
