from encoder import EncoderFile
import pmm_config
from os import getpid, waitpid, kill
from osutil import unlink_nothrow
from sys import argv, exit, stdout
from daemon import daemon, background
from xml.dom import minidom
from session import Session
import stacktrace
from shutil import rmtree
import cStringIO, logging
import threading

from processor import *
from backend_interface import *
import validator
import log, re
from PacketServer import PacketServer, ValidatedPacketHandler, ValidationFailed


_logger = logging.getLogger("pmm.supervisor")


class SupervisorHandler(ValidatedPacketHandler):
    def __init__(self, validator, session, request, client_address, server):
        self.validator = validator
        self.session = session
        ValidatedPacketHandler.__init__(self, request, client_address, server)

    def validateRequest(self, request):
        if not self.session.connectivityInfo and not _logger.isEnabledFor(logging.DEBUG):
            request = re.sub(r'<password>[^<]+</password>', '<password>******</password>', request, 1)

        _logger.info("Incoming packet:\n" + request)

        (ret, errstr) = self.validator.do(request)
        if ret != 0:
            response = ActionResponse()
            response.setCommand_parse_error(StatusCommandParseError(errstr))
            raise ValidationFailed(response)

    def deserializeRequest(self, request):
        action = Action.factory()
        action.build(minidom.parseString(request).childNodes[0])
        return action

    def do(self, request):
        return self.session.processRequest(request)

    def serializeResponse(self, response):
        resp_str = cStringIO.StringIO()
        resp_encoded = EncoderFile(resp_str, "utf-8")
        resp_encoded.write('<?xml version="1.0" encoding="UTF-8"?>')
        response.export(resp_encoded, 0, name_ = 'action-response')

        _logger.info("Outgoing packet:\n" + resp_str.getvalue())
        return resp_str.getvalue()

    def validateResponse(self, response):
        self.validator.do(response)

    def processError(self):
        trace = stacktrace.stacktrace()
        response = ActionResponse.factory()
        response.setCommand_parse_error(StatusCommandParseError("Parse error\n" + trace))
        return response


def usage():
    print "Usage: supervisor <session-path>"

def validate_path(path):
    if not os.path.isabs(path):
        return (False, "Session path must be absolute: " +  path + "\n")

    if os.path.exists(path) and not os.path.isdir(path):
        return (False, "Path " + path + " exists, but not a directory.\n")
    if not os.path.exists(path):
        try:
            os.makedirs(path, 0700)
        except UnicodeError, e:
            return (False, "Invalid character in session directory\n")
        except OSError, e:
            return (False, "Unable to create directory " + path + ": " + e.strerror + "\n")
    return (True, None)

def main():
    debug_mode = os.environ.has_key("DEBUG_SUPERVISOR")
    foreground_mode = os.environ.has_key("DEBUG_SUPERVISOR_FOREGROUND")
    background_mode = os.environ.has_key("PLESKRESTORE_MODE") # not daemon mode with opened stderr

    if len(argv) < 2 or (len(argv) > 2 and not debug_mode):
        usage()
        exit(1)

    session_dir = argv[1]

    (valid, errmsg) = validate_path(session_dir)
    if not valid:
        stdout.write(errmsg)
        exit(2)

    # here ``session_dir`` is present and a directory.

    log.initLog("supervisor", session_dir, debug_mode)

    class SupervisorHandlerCreator(object):
        def __init__(self):
            self.session = None

        def getSession(self):
            if self.session == None:
                self.session = Session(AwaitingStartState(), session_dir)
            return self.session

        def do(self, request, client_address, server):
            SupervisorHandler(v, self.getSession(), request, client_address, server)

    shc = SupervisorHandlerCreator()

    try:
        if debug_mode and len(argv) > 2:
            socket = argv[2]
        else:
            socket = os.path.join(pmm_config.tmp_directory(), "supervisor." + str(getpid()))

        if os.path.exists(socket):
            try:
                unlink_nothrow(socket)
            except OSError, e:
                _logger.critical("Unable to remove stale socket file: " + e.strerror)
                stdout.write("Unable to remove stale socket file: " + e.strerror)
                exit(1)

        v = validator.Validator(pmm_config.backend_protocol_schema())


        server = PacketServer(socket, shc.do, pmm_config.supervisor_socket_gid(), 0775)

        print socket

        if not foreground_mode:
            if background_mode:
                background()
            else:
                daemon()

    except Exception, e:
        stdout.write("Unable to start supervisor: " + str(e.__class__) + " " + str(e) + "\n")
        exit(1)

    try:

        try:
            server.serve_forever()
        finally:
            unlink_nothrow(socket)
    except StopServer:
        pass
    except Exception, e:
        _logger.critical("Runtime error in supervisor: \n" + str(e.__class__) + " " + str(e) + "\n" +
                    stacktrace.stacktrace())
        exit(1)

    session = shc.getSession()

    if session.extractor_pid:
        try:
            pid, st = waitpid(session.extractor_pid, os.WNOHANG)
        except OSError, e:
            # check for OSError 10: No child processes
            if e.errno != 10:
                raise
        else:
            if pid <= 0:
                kill(session.extractor_pid, 9)

    if not session.keepArchive:
        a = threading.enumerate()
        for t in a:
            if t != threading.currentThread():
                t.join()

        if session.importMode:
            for file in ['supervisor.log', 'migration.log', 'migration.result', 'migration.status', 'dump-plesk.xml']:
                unlink_nothrow(session_dir + "/" + file)
        else:
            rmtree(session_dir)

if __name__ == '__main__':
    main()

